package jaredbgreat.dldungeons.planner.mapping;


/*This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/	


import jaredbgreat.dldungeons.builder.Builder;
import jaredbgreat.dldungeons.planner.Dungeon;
import jaredbgreat.dldungeons.rooms.Room;

import java.util.ArrayList;

import net.minecraft.block.Block;
import net.minecraft.world.World;

// More data intensive but perhaps simpler

public class MapMatrix {
	
	public World world;
	public int chunkX, chunkZ, origenX, origenZ;
	
	// map of heights to build at
	public byte[][] ceilY;		// Ceiling height
	public byte[][] floorY;		// Floor Height	
	public byte[][] nCeilY;		// Height of Neighboring Ceiling	
	public byte[][] nFloorY;	// Height of Neighboring Floor
	
	// Blocks referenced against theme data	
	public int[][] ceiling;
	public int[][] wall;
	public int[][] floor;
	public int[][] room;		// The index of the room in an array or ArrayList
	
	// Is it a wall?
	public boolean[][] isWall;	    // Is this coordinate occupied by a wall?
	public boolean[][] isFence;	    // Is this coordinate occupied by a wall?
	public boolean[][] hasLiquid;	// Is floor covered by a liquid block?
	public boolean[][] isDoor;		// Is there a door here?
	
	
	@Override
	public void finalize() throws Throwable {
		world = null;
		super.finalize();
	}
	
	
	public MapMatrix(int width, World world, int chunkX, int chunkZ) {
		this.world = world;
		this.chunkX = chunkX;
		this.chunkZ = chunkZ;
		origenX   = (chunkX * 16) - (width / 2) + 8;
		origenZ   = (chunkZ * 16) - (width / 2) + 8;
		ceilY  	  = new byte[width][width];
		floorY    = new byte[width][width];
		nCeilY 	  = new byte[width][width];
		nFloorY   = new byte[width][width];
		room      = new int[width][width];
		ceiling   = new int[width][width];
		wall	  = new int[width][width];
		floor	  = new int[width][width];
		isWall	  = new boolean[width][width];
		isFence	  = new boolean[width][width];
		hasLiquid = new boolean[width][width];	
		isDoor    = new boolean[width][width];
	}
	
	
	public void build(Dungeon dungeon) {
		//System.out.println("Running map.build(Dungeon dungeon); generating dungeon");
		int shiftX = (chunkX * 16) - (room.length / 2) + 8;
		int shiftZ = (chunkZ * 16) - (room.length / 2) + 8;
		int below;
		for(int i = 0; i < room.length; i++)
			for(int j = 0; j < room.length; j++) {
				//System.out.println("Checking X = " + (shiftX + i) + ", Y = " + (shiftZ + j) + " for dungeon; " 
				//		+ " room #" + room[i][j]);
				if(room[i][j] != 0) {
					 Room theRoom = dungeon.rooms.get(room[i][j]);
					 
					 //Builder.placeBlock(world, shiftX + i, 96, shiftZ +j, Block.glowStone.blockID);
					 
					 // Lower parts of the room
					 if(nFloorY[i][j] < floorY[i][j])
						 for(int k = nFloorY[i][j]; k < floorY[i][j]; k++) 
							 if(noLowDegenerate(theRoom, shiftX + i, k, shiftZ + j))
								 Builder.placeBlock(world, shiftX + i, k, shiftZ + j, wall[i][j]);
					 if(nFloorY[i][j] > floorY[i][j])
						 for(int k = floorY[i][j]; k < nFloorY[i][j]; k++) 
							 if(noLowDegenerate(theRoom, shiftX + i, k, shiftZ + j))
								 Builder.placeBlock(world, shiftX + i, k, shiftZ + j, wall[i][j]);
					 if(noLowDegenerate(theRoom, shiftX + i, floorY[i][j] - 1, shiftZ + j)) { 
						 Builder.placeBlock(world, shiftX + i, floorY[i][j] - 1, shiftZ + j, floor[i][j]);
						 if(dungeon.theme.buildFoundation) {
						 below = nFloorY[i][j] < floorY[i][j] ? nFloorY[i][j] - 1 : floorY[i][j] - 2;
						 while(!Builder.isGroundBlock(world, shiftX + i, below, shiftZ + j)) {
						 		Builder.placeBlock(world, shiftX + i, below, shiftZ + j, dungeon.floorBlock);
						 		below--;
						 		if(below < 0) break;
						 	} 
						}
					 }
					 
					 // Upper parts of the room
					 if(!theRoom.sky 
							 && noHighDegenerate(theRoom, shiftX + i, ceilY[i][j] + 1, shiftZ + j))
						 Builder.placeBlock(world, shiftX + i, ceilY[i][j] + 1, shiftZ + j, ceiling[i][j]);
					 for(int k = floorY[i][j]; k <= ceilY[i][j]; k++)
						 if(!isWall[i][j])Builder.deleteBlock(world, shiftX +i, k, shiftZ + j);
						 else if(noHighDegenerate(theRoom, shiftX + i, k, shiftZ + j))
							 Builder.placeBlock(world, shiftX + i, k, shiftZ + j, wall[i][j]);
					 for(int k = nCeilY[i][j]; k < ceilY[i][j]; k++) 
						 if(noHighDegenerate(theRoom, shiftX + i, k, shiftZ + j))
							 Builder.placeBlock(world, shiftX + i, k, shiftZ + j, wall[i][j]);
					 if(isFence[i][j]) 
						 Builder.placeBlock(world, shiftX + i, floorY[i][j], shiftZ + j, dungeon.fenceBlock);					 
					 if(isDoor[i][j]) {
						 Builder.deleteBlock(world, shiftX + i, floorY[i][j],     shiftZ + j);
						 Builder.deleteBlock(world, shiftX + i, floorY[i][j] + 1, shiftZ + j);
					 }
					 
					 // Liquids
					 if(hasLiquid[i][j] && !isWall[i][j]
							 && !world.isAirBlock(shiftX + i, floorY[i][j] - 1, shiftZ + j)) 
						 Builder.placeBlock(world, shiftX + i, floorY[i][j], shiftZ + j, theRoom.liquidBlock);
					 //System.out.println("One column of the dungeon should be built!");
				}
			}
		dungeon.addSpawners();
	}
	
	
	private boolean noHighDegenerate(Room theRoom, int x, int y, int z) {
		return (!theRoom.degenerate || !world.isAirBlock(x, y, z));
	}
	
	
	private boolean noLowDegenerate(Room theRoom, int x, int y, int z) {
		return (!theRoom.degenerateFloors || !world.isAirBlock(x, y, z));
	}
	
	
	
	

	
/*	
//	public void build(Dungeon dungeon) {
////		//System.out.println("Running map.build(Dungeon dungeon); generating dungeon");
////		int shiftX = (chunkX * 16) + 8;
////		int shiftZ = (chunkZ * 16) + 8;
////		int below;
////		for(int i = 0; i < room.length; i++)
////			for(int j = 0; j < room.length; j++) {
////				//System.out.println("Checking X = " + (shiftX + i) + ", Y = " + (shiftZ + j) + " for dungeon; " 
////				//		+ " room #" + room[i][j]);
////				if(room[i][j] != 0) {
////					 Room theRoom = dungeon.rooms.get(room[i][j]);
////					 
////					 Builder.placeBlock(world, shiftX, 96, shiftZ, Block.glowStone.blockID);
////					 
////					 // Lower parts of the room
////					 if(nFloorY[i][j] < floorY[i][j])
////						 for(int k = nFloorY[i][j]; k < floorY[i][j]; k++) 
////							 if(noLowDegenerate(theRoom, shiftX, 128, shiftZ))
////								 Builder.placeBlock(world, shiftX, 128, shiftZ, wall[i][j]);
////					 if(nFloorY[i][j] > floorY[i][j])
////						 for(int k = floorY[i][j]; k < nFloorY[i][j]; k++) 
////							 if(noLowDegenerate(theRoom, shiftX, 128, shiftZ))
////								 Builder.placeBlock(world, shiftX, 128, shiftZ, wall[i][j]);
////					 if(noLowDegenerate(theRoom, shiftX, 128, shiftZ)) { 
////						 Builder.placeBlock(world, shiftX, 128, shiftZ, floor[i][j]);
////						 if(dungeon.theme.buildFoundation) {
////						 below = nFloorY[i][j] < floorY[i][j] ? nFloorY[i][j] - 1 : floorY[i][j] - 2;
////						 while(!Builder.isGroundBlock(world, shiftX, 128, shiftZ)) {
////						 		Builder.placeBlock(world, shiftX, 128, shiftZ, dungeon.floorBlock);
////						 		below--;
////						 	} 
////						}
////					 }
////					 
////					 // Upper parts of the room
////					 if(!theRoom.sky 
////							 && noHighDegenerate(theRoom, shiftX, ceilY[i][j] + 1, shiftZ))
////						 Builder.placeBlock(world, shiftX, 128, shiftZ, ceiling[i][j]);
////					 for(int k = floorY[i][j]; k <= ceilY[i][j]; k++)
////						 if(!isWall[i][j])Builder.deleteBlock(world, shiftX, 128, shiftZ);
////						 else if(noHighDegenerate(theRoom, shiftX, 128, shiftZ))
////							 Builder.placeBlock(world, shiftX, 128, shiftZ, wall[i][j]);
////					 for(int k = nCeilY[i][j]; k < ceilY[i][j]; k++) 
////						 if(noHighDegenerate(theRoom, shiftX, 128, shiftZ))
////							 Builder.placeBlock(world, shiftX, 128, shiftZ, wall[i][j]);
////					 if(isFence[i][j]) 
////						 Builder.placeBlock(world, shiftX, 128, shiftZ, dungeon.fenceBlock);					 
////					 if(isDoor[i][j]) {
////						 Builder.deleteBlock(world, shiftX, 128,     shiftZ);
////						 Builder.deleteBlock(world, shiftX, 128, shiftZ);
////					 }
////					 
////					 // Liquids
////					 if(hasLiquid[i][j] && !isWall[i][j]
////							 && !world.isAirBlock(shiftX, 128, shiftZ)) 
////						 Builder.placeBlock(world, shiftX, 128, shiftZ, theRoom.liquidBlock);
////					 //System.out.println("One column of the dungeon should be built!");
////				}
////			}
////		dungeon.addSpawners();
//	}
*/
	
}
